// Copyright (c) 2024 Celestino Amoroso (celestino.amoroso@gmail.com).
// All rights reserved.

// context.go
package expr

// ---- Function template
type FuncTemplate func(ctx ExprContext, name string, args []any) (result any, err error)

// ---- Functor interface
type Functor interface {
	Invoke(ctx ExprContext, name string, args []any) (result any, err error)
}

type simpleFunctor struct {
	f FuncTemplate
}

func newSimpleFunctor(f FuncTemplate) *simpleFunctor {
	return &simpleFunctor{f: f}
}

func (functor *simpleFunctor) Invoke(ctx ExprContext, name string, args []any) (result any, err error) {
	return functor.f(ctx, name, args)
}

func (functor *simpleFunctor) ToString(opt FmtOpt) string {
	return "func() {<body>}"
}

// ---- Function Param Info
type ExprFuncParam interface {
	Name() string
	Type() string
	IsOptional() bool
	IsRepeat() bool
	DefaultValue() any
}

// ---- Function Info
type ExprFunc interface {
	Name() string
	MinArgs() int
	MaxArgs() int
	Functor() Functor
	Params() []ExprFuncParam
	ReturnType() string
}

// ----Expression Context
type ExprContext interface {
	Clone() ExprContext
	GetVar(varName string) (value any, exists bool)
	SetVar(varName string, value any)
	UnsafeSetVar(varName string, value any)
	EnumVars(func(name string) (accept bool)) (varNames []string)
	EnumFuncs(func(name string) (accept bool)) (funcNames []string)
	GetFuncInfo(name string) (item ExprFunc, exists bool)
	Call(name string, args []any) (result any, err error)
	// RegisterFunc(name string, f Functor, minArgs, maxArgs int)
	RegisterFuncInfo(info ExprFunc)
	RegisterFunc2(name string, f Functor, returnType string, param []ExprFuncParam) error
}
